@@ -15,7 +15,9 @@ R_CallMethodDef CallMethods[] = {
   { "mgcv_RPPt",(DL_FUNC)&mgcv_RPPt,3},
   { "mgcv_Rpchol",(DL_FUNC)&mgcv_Rpchol,4},
   { "mgcv_Rpforwardsolve",(DL_FUNC)&mgcv_Rpforwardsolve,3},
+  { "mgcv_Rpbacksolve",(DL_FUNC)&mgcv_Rpbacksolve,3},
   { "mgcv_Rpcross",(DL_FUNC)&mgcv_Rpcross,3},
+  { "mgcv_madi",(DL_FUNC)&mgcv_madi,4},
   {NULL, NULL, 0}
 };
 
@@ -48,6 +50,7 @@ R_CMethodDef CEntries[] = {
     {"R_cond",(DL_FUNC) &R_cond,5} ,
     {"pls_fit1",(DL_FUNC)&pls_fit1,14},
     {"tweedious",(DL_FUNC)&tweedious,13},
+    {"tweedious2",(DL_FUNC)&tweedious2,13},
     {"psum",(DL_FUNC)&psum,4},
     {"get_detS2",(DL_FUNC)&get_detS2,12},
     {"get_stableS",(DL_FUNC)&get_stableS,14},
@@ -82,4 +85,7 @@ void R_init_mgcv(DllInfo *dll)
 {
     R_registerRoutines(dll, CEntries, CallMethods, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
+    R_RegisterCCallable("mgcv","mgcv_pmmult2", (DL_FUNC) &mgcv_pmmult2); 
+    R_RegisterCCallable("mgcv","pls_fit1", (DL_FUNC) &pls_fit1);
+    R_RegisterCCallable("mgcv","gdi2", (DL_FUNC) &gdi2); 
 }
