@@ -150,7 +150,10 @@ SEXP mgcv_Rpchol(SEXP Amat,SEXP PIV,SEXP NT,SEXP NB);
 void dchol(double *dA, double *R, double *dR,int *p);
 void vcorr(double *dR,double *Vr,double *Vb,int *p,int *M);
 SEXP mgcv_Rpforwardsolve(SEXP R, SEXP B,SEXP NT);
+SEXP mgcv_Rpbacksolve(SEXP R, SEXP B,SEXP NT);
 SEXP mgcv_Rpcross(SEXP A, SEXP NT,SEXP NB);
+SEXP mgcv_madi(SEXP a, SEXP b,SEXP ind,SEXP diag);
+
 
 /* basis constructor/prediction routines*/
 
