@@ -1030,7 +1030,8 @@ void kba_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,
   k_nn_work(kd,X,dist,ni,&nn,d,&d2k); /* get 2d+k nearest neighbours */
   
   /* d0 = average of distance to 2d+k nearest neighbours - a useful basic length scale */
-  for (d0=0.0,p=dist,p1=dist+ *n * d2k;p<p1;p++) d0 += *p;d0 /= *n * d2k;
+  for (d0=0.0,p=dist,p1=dist+ *n * d2k;p<p1;p++) d0 += *p;
+  d0 /= *n * d2k;
 
   x = (double *)CALLOC((size_t) *d,sizeof(double));
   /* need to get typical box scale */ 
@@ -1299,7 +1300,7 @@ void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
    in X. 
 
    Each row of n by d matrix X is a point. 
-   ni is a list indices of neighbours.
+   ni is a list of indices of neighbours.
    off (length n) indicates where, in ne, the neighbours of each point lie.
    i.e. ni[0:(off[i]-1)] contains indices neighbours of point 0.
         ni[off[i-1]:(off[i]-1)] contains indices of neighbours of i>1.
@@ -1315,7 +1316,9 @@ void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
    more/fewer points in neighbourhood than are required for FD approximation.
 
    Set up is general to allow for future extension of this routine, but currently 
-   only the d==2, m=3, k=6 TPS like case is dealt with here. 
+   only the d==2, m=3, k=6 TPS like case is dealt with herem where d is dimension
+   m is number of components in penalty and k is number of polynomial coefficients
+   in polynomial from which derivatives are estimated. 
 
 */
   int i,j,k,true=1,kk,l,i0,i1,max_nn=0,jj,di,doff;
